@@ -1,13 +1,19 @@
-#ifndef _GIVARO_INTERNAL_CONFIG_H_
-#define _GIVARO_INTERNAL_CONFIG_H_ 1
 // ==========================================================================
 // $Source: /var/lib/cvs/Givaro/src/kernel/system/givconfig.h,v $
-// Copyright(c)'94-97 by Givaro Team
-// see the copyright file.
+// Copyright(c)'1994-2009 by The Givaro group
+// This file is part of Givaro.
+// Givaro is governed by the CeCILL-B license under French law
+// and abiding by the rules of distribution of free software.
+// see the COPYRIGHT file for more details.
 // Authors: T. Gautier
-// $Id: givconfig.h,v 1.8 2007-10-02 14:37:45 jgdumas Exp $
+// $Id: givconfig.h,v 1.22 2011-02-04 14:50:07 bboyer Exp $
 // ==========================================================================
-// Description: configuration file for Givaro
+/** @file givconfig.h
+ * @ingroup system
+ * @brief configuration file for Givaro
+ */
+#ifndef _GIVARO_INTERNAL_CONFIG_H_
+#define _GIVARO_INTERNAL_CONFIG_H_ 1
 
 
 // * GIVARO_HAVE_ANSI_NAMESPACE:
@@ -23,8 +29,8 @@
 //   as based classes for exceptions.
 // * GIVARO_HAVE_ANSI_LIBRARY:
 //   value: defined/undefined
-//   purpose: define it in order to use ANSI library utilities (except for 
-//   container) as allocation without throwing exception, new str stream, 
+//   purpose: define it in order to use ANSI library utilities (except for
+//   container) as allocation without throwing exception, new str stream,
 //   std header files...
 // * GIVARO_HAVE_LBLAS:
 //   value: defined/undefined
@@ -58,57 +64,128 @@
 // - xx: major version number
 // - yy: minor version number
 // - zz: revision number
-#define GIVARO_VERSION  0x030103
+#define GIVARO_MAJOR_VERSION    3
+#define GIVARO_MINOR_VERSION    7
+#define GIVARO_REVISION_VERSION 0
+#define GIVARO_VERSION          30700
 
 // -- Defines this value both to compile the library of user program
 // value: integer that defines debug level trace information (not well defined)
-//#define GIVARO_DEBUG 1
+#ifdef DEBUG
+#define GIVARO_DEBUG 1
+#endif
 
 
 // ==========================================================================
 // -- Defines the basic integer arithmetics available on this machine
 #include <givaro-config.h>
+#ifdef __GIVARO_HAVE_STDINT_H
+#ifndef __STDC_LIMIT_MACROS
+#define __STDC_LIMIT_MACROS
+#endif
+#include <stdint.h>
+#ifndef INT64_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define INT64_MAX std::numeric_limits<int64_t>::max()
+#endif
+
+#ifndef UINT64_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define UINT64_MAX std::numeric_limits<uint64_t>::max()
+#endif
+
+#ifndef INT32_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define INT32_MAX std::numeric_limits<int32_t>::max()
+#endif
+
+#ifndef UINT32_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define UINT32_MAX std::numeric_limits<uint32_t>::max()
+#endif
+
+#ifndef INT16_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define INT16_MAX std::numeric_limits<int16_t>::max()
+#endif
+
+#ifndef UINT16_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define UINT16_MAX std::numeric_limits<uint16_t>::max()
+#endif
+
+#ifndef INT8_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define INT8_MAX std::numeric_limits<int8_t>::max()
+#endif
+
+#ifndef UINT8_MAX
+#pragma message "#warning somebody nasty previously included <stdint.h> without __STDC_LIMIT_MACROS :)"
+#include <limits>
+#define UINT8_MAX std::numeric_limits<uint8_t>::max()
+#endif
+
+
+#endif
+
 
 # define GIVARO_BITS_PER_LONGINT 	SIZEOF_LONG
 # define GIVARO_BITS_PER_INT		SIZEOF_INT
 # define GIVARO_BITS_PER_SHORTINT	SIZEOF_SHORT
 # define GIVARO_BITS_PER_CHAR		SIZEOF_CHAR
 
-typedef signed    __GIVARO_INT8       int8;
-typedef signed    __GIVARO_INT16      int16;
-typedef signed 	  __GIVARO_INT32      int32;
-typedef unsigned  __GIVARO_INT8      uint8;
-typedef unsigned  __GIVARO_INT16     uint16;
-typedef unsigned  __GIVARO_INT32     uint32;
+#if 0 /*  standard types should be used */
+#ifndef __GIVARO_HAVE_STDINT_H
+typedef signed    __GIVARO_INT8      int8_t;
+typedef signed    __GIVARO_INT16     int16_t;
+typedef signed 	  __GIVARO_INT32     int32_t;
+typedef unsigned  __GIVARO_INT8      uint8_t;
+typedef unsigned  __GIVARO_INT16     uint16_t;
+typedef unsigned  __GIVARO_INT32     uint32_t;
 
 # if defined(__GIVARO_INT64)
-#   undef GIVARO_DONOTUSE_SIXTYFOUR 
-typedef signed __GIVARO_INT64 int64;
-typedef unsigned __GIVARO_INT64  uint64;
+#   undef GIVARO_DONOTUSE_SIXTYFOUR
+typedef signed    __GIVARO_INT64     int64_t;
+typedef unsigned  __GIVARO_INT64     uint64_t;
 # else
 #   undef GIVARO_USE_SIXTYFOUR
 #   define GIVARO_DONOTUSE_SIXTYFOUR 1
 # endif
+#endif
+#endif
 
 
 
-# define GIVARO_MAXUINT8		255U 		// 2^8-1
-# define GIVARO_MAXUINT16		65535U 		// 2^16-1
+#ifndef __GIVARO_HAVE_STDINT_H
+# define GIVARO_MAXUINT8		255U 		    // 2^8-1
+# define GIVARO_MAXUINT16		65535U 		    // 2^16-1
 # define GIVARO_MAXUINT32		4294967295U 	// 2^32-1
 # define GIVARO_MAXULONG		4294967295U 	// 2^32-1
-
+#else
+# define GIVARO_MAXUINT8		UINT8_MAX 	    // 2^8-1
+# define GIVARO_MAXUINT16		UINT16_MAX 		// 2^16-1
+# define GIVARO_MAXUINT32		UINT32_MAX 	    // 2^32-1
+# define GIVARO_MAXULONG		4294967295U 	//! @bug 2^32-1
+#endif
 
 
 // ==========================================================================
 // -- Code expansion depending on the previous defined macros
 
 
-// -- specialized template 
+// -- specialized template
 
 #ifdef GIVARO_HAVE_ANSI_SPECIALIZED
-#  define GIVARO_SPECIALIZED template<> 
+#  define GIVARO_SPECIALIZED template<>
 #else
-#  define GIVARO_SPECIALIZED 
+#  define GIVARO_SPECIALIZED
 #endif
 
 
@@ -131,34 +208,43 @@ typedef unsigned __GIVARO_INT64  uint64;
 #    include <sstream>
 #    define GIVARO_ASSERT(cond, msg) { \
       if (!(cond)) {\
-        ostringstream ostr;\
+        std::ostringstream ostr;\
         ostr << msg << "\nFile:"##__FILE__##", Line:" << __LINE__;\
         GivError::throw_error( GivError(ostr.str().c_str()) );\
       }}
 #    define GIVARO_ASSERT2(cond, msg1, msg2) { \
       if (!(cond)) {\
-        ostringstream ostr;\
+        std::ostringstream ostr;\
         ostr << msg1 << msg2 << "\nFile:"##__FILE__##", Line:" << __LINE__;\
         GivError::throw_error( GivError(ostr.str().c_str()) );\
       }}
 #  else
-#    include <strstream>
+#    include <sstream>
 #    define GIVARO_ASSERT(cond, msg) { \
       if (!(cond)) {\
-        ostrstream ostr;\
+        std::ostringstream ostr;\
         ostr << msg << "\nFile:" << __FILE__ << ", Line:" << __LINE__;\
-        GivError::throw_error( GivError(ostr.str()) );\
+        GivError::throw_error( GivError(ostr.str().c_str()) );\
       }}
 #    define GIVARO_ASSERT2(cond, msg1, msg2) { \
       if (!(cond)) {\
-        ostrstream ostr;\
+        std::ostringstream ostr;\
         ostr << msg1 << msg2 << "\nFile:" << __FILE__ << ", Line:" << __LINE__;\
-        GivError::throw_error( GivError(ostr.str()) );\
+        GivError::throw_error( GivError(ostr.str().c_str()) );\
       }}
 #  endif
+
+#define GIVARO_REQUIRE(bool_expression, msg) GIVARO_ASSERT2(bool_expression, "Givaro required: ", msg)
+#define GIVARO_ENSURE(bool_expression, msg) GIVARO_ASSERT2(bool_expression, "Givaro ensured: ", msg)
+#define GIVARO_STATE(expression)  expression
 #else
 #define GIVARO_ASSERT(cond, msg)
 #define GIVARO_ASSERT2(cond, msg1, msg2)
+#define GIVARO_REQUIRE(ignore, msg) ((void) 0)
+#define GIVARO_ENSURE(ignore, msg) ((void) 0)
+// #define GIVARO_STATE(expression) do { (expression) } while (0)
+#define GIVARO_STATE(ignore)  ((void) 0)
+
 #endif
 
 // ==========================================================================
@@ -170,8 +256,9 @@ typedef unsigned __GIVARO_INT64  uint64;
 
 // ==========================================================================
 // -- System variable
+#ifndef GIVARO_SYS
 #define GIVARO_SYS _SYS_UNDEF
-
+#endif
 
 
 // ==========================================================================
@@ -179,10 +266,12 @@ typedef unsigned __GIVARO_INT64  uint64;
 
 // -- Define this macro to store a log of memory address
 // allocated during computation
-//#define GIVARO_MAPMEM 
+//#define GIVARO_MAPMEM
 
 // -- Define this variable to compute statistics about memory usage
+#ifndef GIVARO_STATMEM
 #define GIVARO_STATMEM  1
+#endif
 
 
 
@@ -230,7 +319,7 @@ template<class XXX> struct GIVARO_numeric_limits {
 #  endif
 #endif
 
-
+//! @bug why not use numeric limits from stl ?
 template<> inline float GIVARO_numeric_limits<float>::max() { return FLT_MAX; }
 template<> inline double GIVARO_numeric_limits<double>::max() { return DBL_MAX; }
 template<> inline short GIVARO_numeric_limits<short>::max() { return SHRT_MAX; }
@@ -250,6 +339,7 @@ template<> inline unsigned long long GIVARO_numeric_limits<unsigned long long>::
 
 template<class XXX> struct Signed_Trait : public GIVARO_numeric_limits<XXX> {
     typedef XXX signed_type;
+    typedef XXX unsigned_type;
 };
 
 template<> struct Signed_Trait<float>  : public GIVARO_numeric_limits<float> {
@@ -316,6 +406,7 @@ template<> struct Signed_Trait<unsigned long>  : public GIVARO_numeric_limits<un
     typedef long long signed_type;
     typedef unsigned long long unsigned_type;
   };
-  #endif 
+  #endif
 
 #endif
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
